//+------------------------------------------------------------------+
//|                                        Custom Moving Average.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2004, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Yellow
#property indicator_color2 Magenta
#property indicator_color3 Blue
#property indicator_color4 White

//---- indicator parameters
extern int MA_A_Period=13;
extern int MA_A_Shift=0;
extern int MA_A_Method=1;
extern int MA_A_Width=4;
extern int MA_B_Period=50;
extern int MA_B_Shift=0;
extern int MA_B_Method=0;
extern int MA_B_Width=4;

//---- indicator buffers
double ExtMapBuffer[4];
//----
int ExtCountedBars[4];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   int   draw_begin1; 
   int   draw_begin2;
   string short_name1; 
   string short_name2;

//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
   SetIndexShift(0,MA_A_Shift);
   SetIndexShift(0,MA_B_Shift);


   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   if(MA_A_Period<2) MA_A_Period=13;
   draw_begin1=MA_A_Period-1;
   if(MA_B_Period<2) MA_B_Period=50;
   draw_begin2=MA_B_Period-1;

//---- indicator short name
 
   switch(MA_A_Method)
     {
      case 1 : short_name1="EMA(";  draw_begin1=0; break;
      case 2 : short_name1="SMMA("; break;
      case 3 : short_name1="LWMA("; break;
      default :
         MA_A_Method=0;
         short_name1="SMA(";
     }
   IndicatorShortName(short_name1+MA_A_Period+")");
   SetIndexDrawBegin(0,draw_begin1);
   SetIndexBuffer(0,ExtMapBuffer[1]);
 
      
  switch(MA_B_Method)
     {
      case 1 : short_name2="EMA(";  draw_begin2=0; break;
      case 2 : short_name2="SMMA("; break;
      case 3 : short_name2="LWMA("; break;
      default :
         MA_B_Method=0;
         short_name2="SMA(";
     }
   IndicatorShortName(short_name2+MA_B_Period+")");
   SetIndexDrawBegin(0,draw_begin2);
   
//---- indicator buffers mapping
   SetIndexBuffer(0,ExtMapBuffer[2]);
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   if(Bars<=MA_A_Period) return(0);
   ExtCountedBars[1]=IndicatorCounted();
   if(Bars<=MA_B_Period) return(0);
   ExtCountedBars[2]=IndicatorCounted();
//---- check for possible errors
   if (ExtCountedBars[1]<0) return(-1);
//---- last counted bar will be recounted
   if (ExtCountedBars[1]>0) ExtCountedBars[1]--;
//----
   switch(MA_A_Method)
     {
      case 0 : sma();  break;
      case 1 : ema();  break;
      case 2 : smma(); break;
      case 3 : lwma();
     }
     
   switch(MA_B_Method)
     {
      case 0 : sma();  break;
      case 1 : ema();  break;
      case 2 : smma(); break;
      case 3 : lwma();
     }
//---- done
   return(0);
  }
//+------------------------------------------------------------------+
//| Simple Moving Average                                            |
//+------------------------------------------------------------------+
void sma()
  {
   double sum1=0; 
   double sum2=0;
   int    i1,pos1=Bars-ExtCountedBars[1]-1;
   int    i2,pos2=Bars-ExtCountedBars[2]-1;

//---- initial accumulation
   if(pos1<MA_A_Period) pos1=MA_A_Period;
   for(i1=1;i1<MA_A_Period;i1++,pos1--)
      sum1+=Close[pos1];
   if(pos2<MA_B_Period) pos2=MA_B_Period;
   for(i2=1;i2<MA_B_Period;i2++,pos2--)
      sum2+=Close[pos2];
//---- main calculation loop
   while(pos1>=0)
     {
      sum1+=Close[pos1];
      ExtMapBuffer[pos1]=sum1/MA_A_Period;
	   sum1-=Close[pos1+MA_A_Period-1];
 	   pos1--;
     }
     
   while(pos2>=0)
     {
      sum2+=Close[pos2];
      ExtMapBuffer[pos2]=sum2/MA_B_Period;
	   sum2-=Close[pos2+MA_B_Period-1];
 	   pos2--;
     }
//---- zero initial bars
   if(ExtCountedBars[1]<1)
      for(i1=1;i1<MA_A_Period;i1++) ExtMapBuffer[Bars-i1]=0;
   if(ExtCountedBars[2]<1)
      for(i2=1;i2<MA_B_Period;i2++) ExtMapBuffer[Bars-i2]=0;
  }
//+------------------------------------------------------------------+
//| Exponential Moving Average                                       |
//+------------------------------------------------------------------+
void ema()
  {
   double pr1=2.0/(MA_A_Period+1);
   int    pos1=Bars-2;
   if(ExtCountedBars[1]>2) pos1=Bars-ExtCountedBars[1]-1;
   
   double pr2=2.0/(MA_B_Period+1);
   int    pos2=Bars-2;
   if(ExtCountedBars[2]>2) pos1=Bars-ExtCountedBars[2]-1;

//---- main calculation loop
   while(pos1>=0)
     {
      if(pos1==Bars-2) ExtMapBuffer[pos1+1]=Close[pos1+1];
      ExtMapBuffer[pos1]=Close[pos1]*pr1+ExtMapBuffer[pos1+1]*(1-pr1);
 	   pos1--;
     }
     
   while(pos2>=0)
     {
      if(pos2==Bars-2) ExtMapBuffer[pos2+1]=Close[pos2+1];
      ExtMapBuffer[pos2]=Close[pos2]*pr2+ExtMapBuffer[pos2+1]*(1-pr2);
 	   pos2--;
     }
  }
//+------------------------------------------------------------------+
//| Smoothed Moving Average                                          |
//+------------------------------------------------------------------+
void smma()
  {
   double sum1=0;
   int    i1,k1,pos1=Bars-ExtCountedBars[1]+1;
//---- main calculation loop
   pos1=Bars-MA_A_Period;
   if(pos1>Bars-ExtCountedBars[1]) pos1=Bars-ExtCountedBars[1];
   while(pos1>=0)
     {
      if(pos1==Bars-MA_A_Period)
        {
         //---- initial accumulation
         for(i1=0,k1=pos1;i1<MA_A_Period;i1++,k1++)
           {
            sum1+=Close[k1];
            //---- zero initial bars
            ExtMapBuffer[k1]=0;
           }
        }
      else sum1=ExtMapBuffer[pos1+1]*(MA_A_Period-1)+Close[pos1];
      ExtMapBuffer[pos1]=sum1/MA_A_Period;
 	   pos1--;
     }

   double sum2=0;
   int    i2,k2,pos2=Bars-ExtCountedBars[2]+1;
//---- main calculation loop     
   pos2=Bars-MA_B_Period;
   if(pos2>Bars-ExtCountedBars[2]) pos1=Bars-ExtCountedBars[2];
   while(pos2>=0)
     {
      if(pos2==Bars-MA_B_Period)
        {
         //---- initial accumulation
         for(i2=0,k2=pos2;i2<MA_B_Period;i2++,k2++)
           {
            sum2+=Close[k2];
            //---- zero initial bars
            ExtMapBuffer[k2]=0;
           }
        }
      else sum2=ExtMapBuffer[pos2+1]*(MA_B_Period-1)+Close[pos2];
      ExtMapBuffer[pos2]=sum2/MA_B_Period;
 	   pos2--;
     }
  }
//+------------------------------------------------------------------+
//| Linear Weighted Moving Average                                   |
//+------------------------------------------------------------------+
 void lwma()
  {
   double sum1=0.0,lsum1=0.0;
   double price1;
   int    i1,weight1=0,pos1=Bars-ExtCountedBars[1]-1;
//---- initial accumulation
   if(pos1<MA_A_Period) pos1=MA_A_Period;
   for(i1=1;i1<=MA_A_Period;i1++,pos1--)
     {
      price1=Close[pos1];
      sum1+=price1*i1;
      lsum1+=price1;
      weight1+=i1;
     }
     
   double sum2=0.0,lsum2=0.0;
   double price2;
   int    i2,weight2=0,pos2=Bars-ExtCountedBars[2]-1;
     
   if(pos2<MA_B_Period) pos2=MA_B_Period;
   for(i2=1;i2<=MA_B_Period;i2++,pos2--)
     {
      price2=Close[pos2];
      sum2+=price2*i2;
      lsum2+=price2;
      weight2+=i2;
     }
//---- main calculation loop
   pos1++;
   i1=pos1+MA_A_Period;
   while(pos1>=0)
     {
      ExtMapBuffer[pos1]=sum1/weight1;
      if(pos1==0) break;
      pos1--;
      i1--;
      price1=Close[pos1];
      sum1=sum1-lsum1+price1*MA_A_Period;
      lsum1-=Close[i1];
      lsum1+=price1;
     }
     
   pos2++;
   i2=pos2+MA_B_Period;
   while(pos2>=0)
     {
      ExtMapBuffer[pos2]=sum2/weight2;
      if(pos2==0) break;
      pos2--;
      i2--;
      price2=Close[pos2];
      sum2=sum2-lsum2+price2*MA_B_Period;
      lsum2-=Close[i2];
      lsum2+=price2;
     }
//---- zero initial bars
   if(ExtCountedBars[1]<1)
      for(i1=1;i1<MA_A_Period;i1++) ExtMapBuffer[Bars-i1]=0;
      
   if(ExtCountedBars[2]<1)
      for(i2=1;i2<MA_B_Period;i2++) ExtMapBuffer[Bars-i2]=0;
  }
//+------------------------------------------------------------------+

